///////////////////////////////////////////////////////////
// Company         : Micro Digital                       //
// Address         : Office # C7 Raza Plaza              //
//                   DAV College Road                    //
//                   Rawalpindi,                         //
//                   Pakistan.                           //
// Programmed By   : Rashid Mehmood                      //
// Project Name    : 8051 4 Digit Electronic Code Lock   //
//                   with LCD and Keypad                 //
// Crystal         : 24.000000 MHz                       //
// Microcontroller : AT89C51-C52-C55-S51-S52             //
///////////////////////////////////////////////////////////

#include <AT89x51.h>
//#include "User Interface.h"
#include "lcd4bit.h"
#include "keypad.h"

#define Lock P2_0

#define MAX_IN_CHARS 4

#define PASSCODE 1234

// This functions reads 4 digit
// passcode input from keypad.
int GetPasscode()
{
	unsigned char Ctr = 0;
	char CurChar;
	int Value = -1;
	gotoxy_lcd(1, 1);
	print_lcd("Enter Password  \0");
	do
	{
		gotoxy_lcd(1, 2);
		print_lcd("____            \0");
		gotoxy_lcd(1, 2);
		do
		{
			CurChar = kgetch();
			if(CurChar >= '0' && CurChar <= '9')
			{
				if(Value == -1)
					Value = 0;
				Value *= 10;
				Value += CurChar - '0';
				putc_lcd('*');
				Ctr ++;
			}
		}
		while(CurChar != '*' && CurChar != '#' && Ctr < MAX_IN_CHARS);
	}
	while(CurChar == '*');
	return Value;
}

// This function shows
// lock opened message
// to user.
void ShowOpened()
{
	gotoxy_lcd(1, 1);
	print_lcd("Opened. Press * \0");
	gotoxy_lcd(1, 2);
	print_lcd("key to close... \0");
}

// This function shows
// invalid passcode
// message to user.
void ShowError()
{
	gotoxy_lcd(1, 1);
	print_lcd("Invalid Passcode\0");
	gotoxy_lcd(1, 2);
	print_lcd("Press * to retry\0");
}

// Main entry 
// point.
void main()
{
	char key;
	// Set locked.
	Lock = 0;

	// Initialize LCD.
	init_lcd();

	while(1)
	{
		// Get passcode input from user.
		if(GetPasscode() == PASSCODE)
		{
			// Set unlocked.
			Lock = 1;
			// Show lock opened
			// message on LCD.
			ShowOpened();
		}
		else
		{
			// Show invalid
			// passcode message
			// on LCD
			ShowError();
		}

		// Read key until '*'
		// is pressed.
		do
		{
			key = kgetch();
		}
		while(key != '*');
		
		// Set locked.
		Lock = 0;
	}
}

